@@ -40,7 +40,6 @@
 #include "wimlib/file_io.h"
 #include "wimlib/integrity.h"
 #include "wimlib/metadata.h"
-#include "wimlib/ntfs_3g.h" /* for libntfs3g_global_init() */
 #include "wimlib/security.h"
 #include "wimlib/wim.h"
 #include "wimlib/xml.h"
@@ -162,7 +161,7 @@ wimlib_create_new_wim(enum wimlib_compression_type ctype, WIMStruct **wim_ret)
 	int ret;
 	WIMStruct *wim;
 
-	ret = wimlib_global_init(WIMLIB_INIT_FLAG_ASSUME_UTF8);
+	ret = wimlib_global_init(0);
 	if (ret)
 		return ret;
 
@@ -790,7 +789,7 @@ open_wim_as_WIMStruct(const void *wim_filename_or_fd, int open_flags,
 	WIMStruct *wim;
 	int ret;
 
-	ret = wimlib_global_init(WIMLIB_INIT_FLAG_ASSUME_UTF8);
+	ret = wimlib_global_init(0);
 	if (ret)
 		return ret;
 
@@ -936,21 +935,6 @@ wimlib_free(WIMStruct *wim)
 	wim_decrement_refcnt(wim);
 }
 
-static bool
-test_locale_ctype_utf8(void)
-{
-#ifdef __WIN32__
-	return false;
-#else
-	char *ctype = nl_langinfo(CODESET);
-
-	return (strstr(ctype, "UTF-8") ||
-		strstr(ctype, "UTF8") ||
-		strstr(ctype, "utf8") ||
-		strstr(ctype, "utf-8"));
-#endif
-}
-
 /* API function documented in wimlib.h  */
 WIMLIBAPI u32
 wimlib_get_version(void)
@@ -999,19 +983,11 @@ wimlib_global_init(int init_flags)
 		goto out_unlock;
 
 	xml_global_init();
-	if (!(init_flags & WIMLIB_INIT_FLAG_ASSUME_UTF8)) {
-		wimlib_mbs_is_utf8 = test_locale_ctype_utf8();
-	#ifdef WITH_NTFS_3G
-		if (!wimlib_mbs_is_utf8)
-			libntfs3g_global_init();
-	#endif
-	}
 #ifdef __WIN32__
 	ret = win32_global_init(init_flags);
 	if (ret)
 		goto out_unlock;
 #endif
-	iconv_global_init();
 	init_upcase();
 	if (init_flags & WIMLIB_INIT_FLAG_DEFAULT_CASE_SENSITIVE)
 		default_ignore_case = false;
@@ -1038,7 +1014,6 @@ wimlib_global_cleanup(void)
 		goto out_unlock;
 
 	xml_global_cleanup();
-	iconv_global_cleanup();
 #ifdef __WIN32__
 	win32_global_cleanup();
 #endif
